/*****************************************************************************
 *
 *   Copyright(C) 2013, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/

#include "board.h"
#include <string.h>

/*****************************************************************************
 * Private types/enumerations/variables
 ****************************************************************************/

#define DATA_SIZE			15658
#define BUFFER_SIZE			1048 * 4

extern unsigned char audio[];
uint8_t tx_buffer[BUFFER_SIZE];
uint32_t data_offset, buffer_offset,remain_data;
uint32_t tx_offset = 0;
Bool Tx_EndofBuf_F = FALSE;
volatile uint32_t txDone = 1;

/*****************************************************************************
 * Public types/enumerations/variables
 ****************************************************************************/

/*****************************************************************************
 * Private functions
 ****************************************************************************/

static void PrintGreeting(void)
{
	DEBUGOUT("\r\n***************************************************\r\n");
	DEBUGOUT("*                                                 *\r\n");
	DEBUGOUT("* I2S audio test program for LPC4357 OEM Board... *\r\n");
	DEBUGOUT("* (C) Embedded Artists 2001-2013                  *\r\n");
	DEBUGOUT("*                                                 *\r\n");
	DEBUGOUT("*                           " __DATE__ ", " __TIME__ " *\r\n");
	DEBUGOUT("***************************************************\r\n");
}

/*********************************************************************//**
 * @brief		Initialize transmit buffer
 * @param[in]	none
 * @return 		None
 **********************************************************************/
void Buffer_Init(void)
{
	memcpy(tx_buffer, audio, BUFFER_SIZE);
	buffer_offset = 0;
	data_offset = BUFFER_SIZE;
	remain_data = DATA_SIZE - BUFFER_SIZE;
}

/*********************************************************************//**
 * @brief		I2S callback function, will be call when I2S send a half of
 				buffer complete
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void I2S_Callback(void)
{
	if(remain_data >=BUFFER_SIZE/2)
	{
		if(buffer_offset == BUFFER_SIZE)
		{
			// copy audio data into remain half of tx_buffer
			memcpy(tx_buffer + BUFFER_SIZE/2, audio+data_offset, BUFFER_SIZE/2);
			buffer_offset = 0;
		}
		else
			// copy audio data into remain half of tx_buffer
			memcpy(tx_buffer, audio+data_offset, BUFFER_SIZE/2);
			data_offset += BUFFER_SIZE/2;
			remain_data -= BUFFER_SIZE/2;
	}
	else // reach the last copy
	{
		if(buffer_offset == BUFFER_SIZE)
		{
			// copy audio data into remain half of tx_buffer
			memcpy(tx_buffer + BUFFER_SIZE/2, audio+data_offset, remain_data);
			buffer_offset = 0;
		}
		else
			// copy audio data into remain half of tx_buffer
			memcpy(tx_buffer, audio+data_offset, remain_data);

	}
}

/*****************************************************************************
 * Public functions
 ****************************************************************************/

/**
 * @brief	I2S0 interrupt handler sub-routine
 * @return	Nothing
 */
void I2S0_IRQHandler(void)
{
	uint32_t txlevel,i;
	txlevel = Chip_I2S_GetTxLevel(LPC_I2S0);
	if(txlevel <= 4)
	{
		for(i=0;i<8-txlevel;i++)
		{
			//LPC_I2S0->TXFIFO = *(uint32_t *)(tx_buffer + buffer_offset);
			Chip_I2S_Send(LPC_I2S0, *(uint32_t *)(tx_buffer + buffer_offset));
			tx_offset +=4;
			buffer_offset +=4;
			//call I2S_Callback() function when reach each half of tx_buffer
			if((buffer_offset == BUFFER_SIZE/2)||(buffer_offset == BUFFER_SIZE))
				I2S_Callback();
			if(tx_offset >= DATA_SIZE)//reach the end of buffer
			{
				tx_offset = 0;
				Buffer_Init();
				txDone = 1;
				NVIC_DisableIRQ(I2S0_IRQn);
				Chip_I2S_TxStop(LPC_I2S0);
			}
		}
	}
}

/**
 * @brief	Main entry point
 * @return	Nothing
 */
int main(void)
{
	I2S_AUDIO_FORMAT_T audio_Confg;
	SystemCoreClockUpdate();
	Board_Init();

	/* Print the Demo Information */
	PrintGreeting();

	Buffer_Init();

	  /* 		This code will set I2S in MASTER mode, 44100Hz, Mono,16bit
	   * 		This example is used to test I2S transmit mode. There is an audio
	   * 		data in audiodata.c in mono 44100Hz 16 bit, it will be send out to
	   * 		I2S port. User must plug Ext I2S DAC IC to hear sound.
	   */
	audio_Confg.SampleRate = 44100;
	audio_Confg.ChannelNumber = 1;
	audio_Confg.WordWidth = 16;

	Board_Audio_Init(LPC_I2S0, UDA1380_LINE_IN);

	Chip_I2S_Init(LPC_I2S0);
	Chip_I2S_TxConfig(LPC_I2S0, &audio_Confg);

	Chip_I2S_TxStop(LPC_I2S0);
	Chip_I2S_DisableMute(LPC_I2S0);
	Chip_I2S_TxStart(LPC_I2S0);

	Chip_I2S_Int_TxCmd(LPC_I2S0, ENABLE, 4);
	NVIC_EnableIRQ(I2S0_IRQn);

	/* repeat the sound in a loop */
	while(1) {

		if(txDone) {
			txDone = 0;

			NVIC_ClearPendingIRQ(I2S0_IRQn);

			Chip_I2S_TxStart(LPC_I2S0);

			NVIC_EnableIRQ(I2S0_IRQn);
		}
	}

}
