/*****************************************************************************
 *
 *   Copyright(C) 2013, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/

#include "board.h"

/*****************************************************************************
 * Private types/enumerations/variables
 ****************************************************************************/

extern const unsigned char sound_8k[];
extern int sound_sz;

/*****************************************************************************
 * Public types/enumerations/variables
 ****************************************************************************/

/*****************************************************************************
 * Private functions
 ****************************************************************************/

static void PrintGreeting(void)
{
	DEBUGOUT("\r\n***************************************************\r\n");
	DEBUGOUT("*                                                 *\r\n");
	DEBUGOUT("* DAC test program for LPC4357 OEM Board...       *\r\n");
	DEBUGOUT("* (C) Embedded Artists 2001-2013                  *\r\n");
	DEBUGOUT("*                                                 *\r\n");
	DEBUGOUT("*                           " __DATE__ ", " __TIME__ " *\r\n");
	DEBUGOUT("***************************************************\r\n");
}

static void Play(void)
{
  uint32_t cnt = 0;
  uint32_t off = 0;
  uint32_t sampleRate = 0;
  uint32_t delay = 0;

	/* set time out for DAC*/
	Chip_DAC_SetDMATimeOut(LPC_DAC, 0xFFFF);
	Chip_DAC_ConfigDAConverterControl(LPC_DAC, DAC_DMA_ENA);


  /* ChunkID */
  if (sound_8k[cnt] != 'R' && sound_8k[cnt+1] != 'I' &&
      sound_8k[cnt+2] != 'F' && sound_8k[cnt+3] != 'F')
  {
	  DEBUGOUT("Wrong format (RIFF)\r\n");
    return;
  }
  cnt+=4;

  /* skip chunk size*/
  cnt += 4;

  /* Format */
  if (sound_8k[cnt] != 'W' && sound_8k[cnt+1] != 'A' &&
      sound_8k[cnt+2] != 'V' && sound_8k[cnt+3] != 'E')
  {
	  DEBUGOUT("Wrong format (WAVE)\r\n");
    return;
  }
  cnt+=4;

  /* SubChunk1ID */
  if (sound_8k[cnt] != 'f' && sound_8k[cnt+1] != 'm' &&
      sound_8k[cnt+2] != 't' && sound_8k[cnt+3] != ' ')
  {
	  DEBUGOUT("Missing fmt\r\n");
    return;
  }
  cnt+=4;

  /* skip chunk size, audio format, num channels */
  cnt+= 8;

  sampleRate = (sound_8k[cnt] | (sound_8k[cnt+1] << 8) |
          (sound_8k[cnt+2] << 16) | (sound_8k[cnt+3] << 24));

  if (sampleRate != 8000) {
	  DEBUGOUT("Only 8kHz supported\r\n");
    return;
  }

  delay = 1000000 / sampleRate;

  cnt+=4;

  /* skip byte rate, align, bits per sample */
  cnt += 8;

  /* SubChunk2ID */
  if (sound_8k[cnt] != 'd' && sound_8k[cnt+1] != 'a' &&
      sound_8k[cnt+2] != 't' && sound_8k[cnt+3] != 'a')
  {
	  DEBUGOUT("Missing data\r\n");
    return;
  }
  cnt += 4;

  /* skip chunk size */
  cnt += 4;

  off = cnt;

  while(1)
  {
    cnt = off;
    while(cnt++ < sound_sz)
    {
    	Chip_DAC_UpdateValue(LPC_DAC, (sound_8k[cnt]&0x3FF));

      Board_DelayUs(delay);
    }
  }
}

/*****************************************************************************
 * Public functions
 ****************************************************************************/

/**
 * @brief	Main entry point
 * @return	Nothing
 */
int main(void)
{
	SystemCoreClockUpdate();
	Board_Init();
	Board_DAC_Init(LPC_DAC);
	Chip_DAC_Init(LPC_DAC);

	/* Print the Demo Information */
	PrintGreeting();

	Play();

	return 0;
}
