/*****************************************************************************
 *
 *   Copyright(C) 2011, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/

#include "lpc_types.h"
#include "lpc43xx_cgu.h"
#include "lpc43xx_dac.h"
#include "lpc43xx_rgu.h"
#include "lpc43xx_scu.h"
#include "lpc43xx_timer.h"
#include "lpc43xx_uart.h"

#include "board.h"


/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/

/* Buffer size for retrieving the display information */
#define MAIN_BUF_SZ 100

/******************************************************************************
 * Local variables
 *****************************************************************************/

/******************************************************************************
 * External Functions
 *****************************************************************************/

extern const unsigned char sound_8k[];
extern int sound_sz;

/******************************************************************************
 * Local Functions
 *****************************************************************************/

static void PrintGreeting(void)
{
  console_sendString("\r\n***************************************************\r\n");
  console_sendString("*                                                 *\r\n");
  console_sendString("* DAC test program for LPC4357 OEM Board...       *\r\n");
  console_sendString("* (C) Embedded Artists 2001-2013                  *\r\n");
  console_sendString("*                                                 *\r\n");
  console_sendString("*                           " __DATE__ ", " __TIME__ " *\r\n");
  console_sendString("***************************************************\r\n");
}

static void Play(void)
{
  uint32_t cnt = 0;
  uint32_t off = 0;
  uint32_t sampleRate = 0;
  uint32_t delay = 0;
	DAC_CONVERTER_CFG_Type DAC_ConverterConfigStruct;

  DAC_Init(LPC_DAC);
	DAC_ConverterConfigStruct.DBLBUF_ENA = RESET;
	DAC_ConverterConfigStruct.CNT_ENA = RESET;
	DAC_ConverterConfigStruct.DMA_ENA = SET;
	DAC_ConfigDAConverterControl(LPC_DAC, &DAC_ConverterConfigStruct);
  CGU_EnableEntity(CGU_BASE_APB3, ENABLE);

  /* ChunkID */
  if (sound_8k[cnt] != 'R' && sound_8k[cnt+1] != 'I' &&
      sound_8k[cnt+2] != 'F' && sound_8k[cnt+3] != 'F')
  {
    console_sendString((uint8_t*)"Wrong format (RIFF)\r\n");
    return;
  }
  cnt+=4;

  /* skip chunk size*/
  cnt += 4;

  /* Format */
  if (sound_8k[cnt] != 'W' && sound_8k[cnt+1] != 'A' &&
      sound_8k[cnt+2] != 'V' && sound_8k[cnt+3] != 'E')
  {
    console_sendString((uint8_t*)"Wrong format (WAVE)\r\n");
    return;
  }
  cnt+=4;

  /* SubChunk1ID */
  if (sound_8k[cnt] != 'f' && sound_8k[cnt+1] != 'm' &&
      sound_8k[cnt+2] != 't' && sound_8k[cnt+3] != ' ')
  {
    console_sendString((uint8_t*)"Missing fmt\r\n");
    return;
  }
  cnt+=4;

  /* skip chunk size, audio format, num channels */
  cnt+= 8;

  sampleRate = (sound_8k[cnt] | (sound_8k[cnt+1] << 8) |
          (sound_8k[cnt+2] << 16) | (sound_8k[cnt+3] << 24));

  if (sampleRate != 8000) {
    console_sendString((uint8_t*)"Only 8kHz supported\r\n");
    return;
  }

  delay = 1000000 / sampleRate;

  cnt+=4;

  /* skip byte rate, align, bits per sample */
  cnt += 8;

  /* SubChunk2ID */
  if (sound_8k[cnt] != 'd' && sound_8k[cnt+1] != 'a' &&
      sound_8k[cnt+2] != 't' && sound_8k[cnt+3] != 'a')
  {
    console_sendString((uint8_t*)"Missing data\r\n");
    return;
  }
  cnt += 4;

  /* skip chunk size */
  cnt += 4;

  off = cnt;

  while(1)
  {
    cnt = off;
    while(cnt++ < sound_sz)
    {
      DAC_UpdateValue(LPC_DAC, (sound_8k[cnt]&0x3FF));

      TIM_Waitus(delay);
    }
  }
}

/******************************************************************************
 * Main method
 *****************************************************************************/

int main (void)
{
	TIM_TIMERCFG_Type timerCfg;

  SystemInit();
  CGU_Init(BOARD_MAX_SPEED);

  RGU_SoftReset(RGU_SIG_DAC);
  RGU_SoftReset(RGU_SIG_DMA);
  RGU_SoftReset(RGU_SIG_GPIO);
  RGU_SoftReset(RGU_SIG_TIMER0);
  
  // initialize timer
  TIM_ConfigStructInit(TIM_TIMER_MODE, &timerCfg);
  TIM_Init(LPC_TIMER0, TIM_TIMER_MODE, &timerCfg);

  console_init();

  PrintGreeting();

  Play();
  
  while(1);
}


