/*****************************************************************************
 *
 *   Copyright(C) 2011, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "lpc_types.h"
#include "lpc43xx_cgu.h"
#include "lpc43xx_rgu.h"
#include "lpc43xx_rtc.h"
#include "lpc43xx_timer.h"
#include "board.h"

/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/


/******************************************************************************
 * Local variables
 *****************************************************************************/

/* Buffer for printing */
static char buff[512];

static volatile uint32_t mstick = 0;

/******************************************************************************
 * External Functions
 *****************************************************************************/

void SysTick_Handler(void)
{
  mstick++;
}

/******************************************************************************
 * Local Functions
 *****************************************************************************/

static void PrintGreeting(void)
{
  console_sendString("\r\n****************************************************\r\n");
  console_sendString("*                                                  *\r\n");
  console_sendString("* Demo of the RTC on the LPC4357 OEM Board...      *\r\n");
  console_sendString("* (C) Embedded Artists 2001-2013                   *\r\n");
  console_sendString("*                                                  *\r\n");
  sprintf(buff, "*                            %s, %s *\r\n", __DATE__, __TIME__);
  console_sendString((uint8_t*)buff);
  console_sendString("****************************************************\r\n");
}

static uint32_t test_rtc(void)
{
  uint32_t result = TRUE;
  uint32_t timeBefore;
  uint32_t timeAfter;

  RTC_TIME_Type time = {
    0,    /*!< Seconds Register */
    48,   /*!< Minutes Register */
    9,    /*!< Hours Register */
    22,   /*!< Day of Month Register */
    1,    /*!< Day of Week Register */
    128,  /*!< Day of Year Register */
    4,    /*!< Months Register */
    2014, /*!< Years Register */
  };
  
  console_sendString("RTC: Initializing RTC, will take a couple of seconds...\r\n");

  RTC_Init(LPC_RTC);
  RTC_SetFullTime(LPC_RTC, &time);
  RTC_Cmd(LPC_RTC, ENABLE);

  console_sendString("RTC: Time set to 0 seconds, waiting for it to reach 2 seconds...\r\n");
  timeBefore = mstick + 4000;
  while((LPC_RTC->SEC != 2) && (mstick < timeBefore)) {
  }
  if (mstick >= timeBefore) {
    // timed out
    console_sendString("RTC: Timeout. Crystal problems?\r\n");
    return FALSE;
  }
  
  console_sendString("RTC: Waiting for it to reach 3 seconds...\r\n");
  while (LPC_RTC->SEC != 3) {
  }
  timeBefore = mstick;
  while (LPC_RTC->SEC != 5) {
  }
  timeAfter = mstick;
  
  sprintf(buff, "RTC: It took %d msticks for RTC to go from 3 to 5 seconds\r\n", timeAfter-timeBefore);
  console_sendString((uint8_t*)buff);
  
  return result;
}

/******************************************************************************
 * Main method
 *****************************************************************************/



int main (void)
{
	TIM_TIMERCFG_Type timerCfg;

  SystemInit();
  CGU_Init(BOARD_MAX_SPEED);

  RGU_SoftReset(RGU_SIG_EMC);
  RGU_SoftReset(RGU_SIG_GPIO);
  RGU_SoftReset(RGU_SIG_TIMER0);
  
  // initialize timer
  TIM_ConfigStructInit(TIM_TIMER_MODE, &timerCfg);
  TIM_Init(LPC_TIMER0, TIM_TIMER_MODE, &timerCfg);

  console_init();

  // Request systicks every 1 ms
  SysTick_Config(SystemCoreClock/1000);

  PrintGreeting();

  test_rtc();

  while(1){
  }
}


